package main;

import static opengl.GL.*;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lwjgl.BufferUtils;
import org.lwjgl.LWJGLException;
import org.lwjgl.opengl.Display;
import util.Util;

/**
 * Displays and transforms a triangle.
 * @author Sascha Kolodzey, Nico Marniok
 */
public class SimpleTriangle {
    private static int programID;       // shader program id
    private static int triangleVAID;    // vertexarray id of the triangle
    
    public static void main(String[] argv) {
        try {
            init();
            
            createShaderProgram();
            createTriangle();
            render();
            Display.destroy();
        } catch (LWJGLException ex) {
            Logger.getLogger(SimpleTriangle.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    public static void render() throws LWJGLException {
        glClearColor(1.0f, 0.0f, 0.0f, 1.0f);
        while(!Display.isCloseRequested()) {
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
            glUseProgram(programID);
            glBindVertexArray(triangleVAID);
            glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_INT, 0);
            Display.update();
            Display.sync(60);
        }
    }
    
    public static void createShaderProgram() {

		// hier erweitern
        
        glBindAttribLocation(programID, 0, "vs_in_pos");
        glBindAttribLocation(programID, 1, "vs_in_normal");
    }
    
    public static void createTriangle() {
        // vertexbuffer
        FloatBuffer vertexData = BufferUtils.createFloatBuffer(18);
        vertexData.put(new float[] { -0.6f, -0.6f, 0.0f,    // triangle 1, vertex 1
                                     +0.4f, -0.6f, 0.0f,    // triangle 1, vertex 2
                                     -0.6f, +0.4f, 0.0f,    // triangle 1, vertex 3
                                     +0.6f, +0.6f, 0.0f,    // triangle 2, vertex 1
                                     +0.6f, -0.4f, 0.0f,    // triangle 2, vertex 2
                                     -0.4f, +0.6f, 0.0f }); // triangle 2, vertex 3
        vertexData.flip();
        
        // indexbuffer
        IntBuffer indexData = BufferUtils.createIntBuffer(6);
        indexData.put(new int[] {0,1,2, 3,4,5});
        indexData.flip();
                
        triangleVAID = glGenVertexArrays();
        glBindVertexArray(triangleVAID);
        
        int indexBufferID = glGenBuffers();
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, indexBufferID);
        glBufferData(GL_ELEMENT_ARRAY_BUFFER, indexData, GL_STATIC_DRAW);
        
        int vertexBufferID = glGenBuffers();
        glBindBuffer(GL_ARRAY_BUFFER, vertexBufferID);
        glBufferData(GL_ARRAY_BUFFER, vertexData, GL_STATIC_DRAW);
        
         // vs_in_pos
        glEnableVertexAttribArray(0);
        glVertexAttribPointer(0, 3, GL_FLOAT, false, 3*4, 0);
    }
}
